#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <popt.h>
#include "libsmbclient.h"
#include "get_auth_data_fn.h"

enum acl_mode
{
    SMB_ACL_GET,
    SMB_ACL_SET,
    SMB_ACL_DELETE,
    SMB_ACL_MODIFY,
    SMB_ACL_ADD,
    SMB_ACL_CHOWN,
    SMB_ACL_CHGRP
};


int main(int argc, const char *argv[])
{
	int counter;
    int opt;
    int flags;
    int debug = 0;
    int numeric = 0;
    int full_time_names = 0;
    enum acl_mode mode = SMB_ACL_GET;
    static char *the_acl = NULL;
    int ret;
    char *p;
    char *debugstr;
    char path[1024];
    char value[1024];
    poptContext pc;

    struct poptOption long_options[] =
        {
            POPT_AUTOHELP
            {
                "numeric", 'n', POPT_ARG_NONE, &numeric,
                1, "Don't resolve sids or masks to names"
            },
            {
                "debug", 'd', POPT_ARG_INT, &debug,
                0, "Set debug level (0-100)"
            },
            {
                "full_time_names", 'f', POPT_ARG_NONE, &full_time_names,
                1,
                "Use new style xattr names, which include CREATE_TIME"
            },
            {
                "delete", 'D', POPT_ARG_STRING, NULL,
                'D', "Delete an acl", "ACL"
            },
			{
				"set", 'S', POPT_ARG_NONE, NULL,
				'S', "Set acls", "ACLS"
			},
			{
                NULL
            }
        };
    
    setbuf(stdout, NULL);

    pc = poptGetContext("smbcacls", argc, argv, long_options, 0);
    
    poptSetOtherOptionHelp(pc, "smb://server1/share1/filename");
    
    while ((opt = poptGetNextOpt(pc)) != -1) {
        switch (opt) {
        case 'S':
            //the_acl = strdup(poptGetOptArg(pc));
            mode = SMB_ACL_SET;
            break;
            
        case 'D':
            the_acl = strdup(poptGetOptArg(pc));
            mode = SMB_ACL_DELETE;
            break;
        }
    }
    
    /* Make connection to server */
    if(!poptPeekArg(pc)) { 
        poptPrintUsage(pc, stderr, 0);
        return 1;
    }
    
    strcpy(path, poptGetArg(pc));
    
    if (smbc_init(get_auth_data_fn, debug) != 0)
    {
        printf("Could not initialize smbc_ library\n");
        return 1;
    }

    if (full_time_names) {
        SMBCCTX *context = smbc_set_context(NULL);
        smbc_option_set(context, "full_time_names", 1);
    }
    
    /* Perform requested action twice */
    
	for( counter = 0; counter < 2; ++counter )
	{
		printf( "Try number %d\n", counter );
		switch(mode)
		{
		case SMB_ACL_SET:
			flags = 0;
			debugstr = "set attributes";
	        
			//if ((p = strchr(the_acl, ':')) == NULL)
			//{
			//	printf("Missing value.  ACL must be name:value pair\n");
			//	return 1;
			//}

			//*p++ = '\0';
			the_acl = strdup("system.nt_sec_desc.*");
			p = strdup("REVISION:1,OWNER:S-1-5-21-517669159-3449754103-2304587396-1005,GROUP:S-1-5-21-517669159-3449754103-2304587396-513,ACL:S-1-5-21-517669159-3449754103-2304587396-1005:0/16/0x001f01ff,ACL:S-1-5-21-517669159-3449754103-2304587396-1018:0/3/0x001200a9,ACL:S-1-3-0:0/27/0x001f01ff,ACL:S-1-5-18:0/19/0x001f01ff,ACL:S-1-5-32-545:0/18/0x00100004,ACL:S-1-5-32-545:0/18/0x00100002,ACL:S-1-5-32-545:0/19/0x001200a9,ACL:S-1-5-32-544:0/19/0x001f01ff");
	        
			ret = smbc_setxattr(path, the_acl, p, strlen(p), flags);
			if (ret < 0)
			{
				printf("Could not %s for [%s] %d: %s\n",
					   debugstr, path, errno, strerror(errno));
				return 1;
			}
			break;

		case SMB_ACL_DELETE:
			ret = smbc_removexattr(path, the_acl);
			if (ret < 0)
			{
				printf("Could not remove attribute %s for [%s] %d:%s\n",
					   the_acl, path, errno, strerror(errno));
				return 1;
			}
			break;

		default:
			printf("operation not yet implemented\n");
			break;
		}
	}

    return 0;
}
